SystemOrganization addCategory: #'Sudoku-Strategies'!
SystemOrganization addCategory: #'Sudoku-Calculation'!
SystemOrganization addCategory: #'Sudoku-GUI'!
SystemOrganization addCategory: #'Sudoku-Core'!

RectangleMorph subclass: #SudokuCell
	instanceVariableNames: 'sudokuModel xCoord yCoord'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Sudoku-GUI'!

SudokuCell subclass: #SudokuCandidateCell
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Sudoku-GUI'!

!SudokuCandidateCell class methodsFor: 'instance creation' stamp: 'MR 2/1/2013 12:02:18.02'!
new
	"This method creates a Sudoku cell that shows all candidates from 1 to 9."
	| candidateCell |
	candidateCell := SudokuSingleNumberCell new initializeCellSize: 25 textSize: 18 
													updateColor: Color black 
													normalColor: Color black
													highlightColor: Color green
													removeColor: (Color r: 1 g:0 b:0).
	^ self newFrom: ((SudokuBuilder createBoxFromMorph: candidateCell) borderWidth: 1)! !

!SudokuCandidateCell methodsFor: 'visual properties' stamp: 'MG 12/15/2012 16:31'!
highlightCandidates: aCollection
"aCollection contains all candidates that should be highlighted."
	aCollection do: 
		[:candidate | ((self submorphs at: candidate-1 \\ 3 + 1) 
								submorphs at: candidate-1 / 3 + 1) 
								updateContent: candidate ofType: #highlight].! !

!SudokuCandidateCell methodsFor: 'visual properties' stamp: 'MG 12/15/2012 16:44'!
markCandidatesToRemove: aCollection
	aCollection do: 
		[:candidate | ((self submorphs at: candidate-1 \\ 3 + 1) 
								submorphs at: candidate-1 / 3 + 1) 
								updateContent: candidate ofType: #removeCandidates].! !

!SudokuCandidateCell methodsFor: 'converting' stamp: 'mg 6/19/2014 11:24'!
transformCandidateToSingleNumber: aNumber
	| singleNumber |
	singleNumber := SudokuSolvedCell new.
	aNumber ~= 0 ifTrue: [singleNumber submorphs first updateColor: Color black].
	singleNumber updateContent: aNumber ofType: #number.
	
	owner := self owner replaceSubmorph: self by: singleNumber.
	self delete.! !

!SudokuCandidateCell methodsFor: 'updating' stamp: 'MR 12/9/2012 23:46'!
updateCandidates: aCollection
	self submorphs do:
		[:threeCandidates | threeCandidates submorphs do:
			[:candidate | candidate updateContent: ' ' ofType: #number]].
	
	aCollection do: 
		[:candidate | ((self submorphs at: candidate-1 \\ 3 + 1) 
								submorphs at: candidate-1 / 3 + 1) 
								updateContent: candidate ofType: #number].! !

!SudokuCandidateCell methodsFor: 'updating' stamp: 'mg 6/18/2014 12:25'!
updateContent: content ofType: aSymbol
	(aSymbol = #number and: [(content ~= '')])
		ifTrue: [self transformCandidateToSingleNumber: content].
	(aSymbol = #candidates)
		ifTrue: [self updateCandidates: content].
	(aSymbol = #highlight)
		ifTrue: [self highlightCandidates: content].
	(aSymbol = #removeCandidates)
		ifTrue: [self markCandidatesToRemove: content].
	self changed.! !

!SudokuCell methodsFor: 'updating' stamp: 'mg 6/23/2014 13:55'!
deleteNumberInSudokuModel

	self sudokuModel disableEqualityConstraintAt: self xCoord at: self yCoord.! !

!SudokuCell methodsFor: 'initialization' stamp: 'mg 6/17/2014 17:58'!
initialize

	super initialize.
	xCoord := 0.
	yCoord := 0.! !

!SudokuCell methodsFor: 'as yet unclassified' stamp: 'mg 6/18/2014 14:19'!
moveFocusBy: aPoint event: anEvent

	| newFocusedCell textMorph |
	newFocusedCell := self sudokuModel sudokuGui sudokuCellAtX: self xCoord + aPoint x Y: self yCoord + aPoint y.
	newFocusedCell ifNil: [^ self].
	textMorph := newFocusedCell submorphs first.
	anEvent hand newKeyboardFocus: textMorph.
	textMorph
		handleInteraction: [textMorph editor mouseDown: anEvent]
		fromEvent: anEvent.! !

!SudokuCell methodsFor: 'accessing' stamp: 'mg 6/17/2014 18:07'!
sudokuModel

	^ sudokuModel! !

!SudokuCell methodsFor: 'accessing' stamp: 'mg 6/17/2014 18:07'!
sudokuModel: anObject

	sudokuModel := anObject! !

!SudokuCell methodsFor: 'updating' stamp: 'mg 6/19/2014 13:40'!
suggestNewValueToSudokuModel: aValue

	self sudokuModel createConstraintAt: self xCoord at: self yCoord with: aValue.! !

!SudokuCell methodsFor: 'updating' stamp: 'MG 12/7/2012 15:17'!
updateContent: content ofType: aSymbol
	self subclassResponsibility.! !

!SudokuCell methodsFor: 'accessing' stamp: 'mg 6/17/2014 17:58'!
xCoord

	^ xCoord! !

!SudokuCell methodsFor: 'accessing' stamp: 'mg 6/17/2014 17:58'!
xCoord: anObject

	xCoord := anObject! !

!SudokuCell methodsFor: 'accessing' stamp: 'mg 6/17/2014 17:58'!
yCoord

	^ yCoord! !

!SudokuCell methodsFor: 'accessing' stamp: 'mg 6/17/2014 17:58'!
yCoord: anObject

	yCoord := anObject! !

SudokuCell subclass: #SudokuSingleNumberCell
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Sudoku-GUI'!

!SudokuSingleNumberCell methodsFor: 'initialization' stamp: 'mg 6/19/2014 11:20'!
initialize
	"This method initializes all 81 Sudoku cells."
	super initialize.
	self initializeCellSize: 77 
		 textSize: 36 
		 updateColor: Color blue 
		 normalColor: Color black 
		 highlightColor: Color green 
		 removeColor: Color blue. 
	self 
		borderWidth: 1.! !

!SudokuSingleNumberCell methodsFor: 'initialization' stamp: 'MR 2/1/2013 12:07:18.91'!
initializeCellSize: cellSize textSize: textSize updateColor: aColor1 normalColor: aColor2 highlightColor: aColor3 removeColor: aColor4
	| textMorph |
	super initialize.
	self
		layoutPolicy: TableLayout new;
		color: Color white;
		borderWidth: 0;
		width: cellSize;
		height: cellSize;
		disableDragNDrop;
		listCentering: #center;
		wrapCentering: #center.
		
	textMorph := SudokuTextMorph new
		borderWidth: 0; 
		width: cellSize/2;
		height: cellSize/2;
		beAllFont: (((TextStyle named: 'BitstreamVeraSans') fontOfSize: textSize) emphasized: 1);
		disableDragNDrop;
		setNumericValue: 0;
		normalColor: aColor2;
		updateColor: aColor1;
		highlightColor: aColor3;
		removeCandidatesColor: aColor4;
		centered.

	self addMorph: textMorph.! !

!SudokuSingleNumberCell methodsFor: 'updating' stamp: 'mg 1/10/2013 16:52:40.749'!
updateContent: candidate ofType: aSymbol
	(aSymbol = #number)
		ifTrue: [self submorphs first updateTextWith: candidate; setToWhiteBackground].
	(aSymbol = #highlight)
		ifTrue: [self submorphs first setToNormalColor; setToHighlightColor].
	(aSymbol = #removeCandidates)
		ifTrue: [self submorphs first setToRemoveCandidatesColor].
	(aSymbol = #candidates)
		ifTrue: [self submorphs first setToWhiteBackground].  
	self changed.! !

SudokuSingleNumberCell subclass: #SudokuSolvedCell
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Sudoku-GUI'!

!SudokuSolvedCell methodsFor: 'updating' stamp: 'mg 2/2/2013 20:52:43.923'!
transformSolvedCellToCandidate: aCollection
	| cell |
	cell := SudokuCandidateCell new.
	cell updateCandidates: aCollection.
	
	owner := self owner replaceSubmorph: self by: cell.
	self delete.! !

!SudokuSolvedCell methodsFor: 'updating' stamp: 'mg 6/18/2014 12:40'!
updateContent: solvedNumber ofType: aSymbol
	(aSymbol = #number)
		ifTrue: [(solvedNumber = 0)
					ifTrue: [self submorphs first updateTextWith: '']
					ifFalse: [self submorphs first updateTextWith: solvedNumber]].
	(aSymbol = #highlight)
		ifTrue: [self submorphs first setToNormalColor.
				(solvedNumber notEmpty) 
					ifTrue: [self submorphs first setToHighlightColor]].
	(aSymbol = #removeCandidates)
		ifTrue: ["do nothing because here are no candidates"].
	(aSymbol = #candidates)
		ifTrue: ["In historyMode, it is possible that solvedNumber is a collection of multiple candidiates. In that case, transform the solved cell back into a candidate cell."
			(solvedNumber isCollection and: [solvedNumber size > 1]) 
				ifTrue: [self transformSolvedCellToCandidate: solvedNumber].  
			self submorphs first setToWhiteBackground].  
	self changed.! !

RectangleMorph subclass: #SudokuStrategyDisplay
	instanceVariableNames: 'textLines'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Sudoku-GUI'!

!SudokuStrategyDisplay methodsFor: 'initialization' stamp: 'MR 1/10/2013 16:55'!
createLine: aString
	| line |
	
	line := TextMorph new.
	line
		beAllFont: (((TextStyle named: 'BitstreamVeraSans') fontOfSize: 24) emphasized: 1);
		contents: aString;
		backgroundColor: Color white;
		autoFit: False;
		hResizing: #spaceFill;
		disableDragNDrop.
	
	textLines add: line.! !

!SudokuStrategyDisplay methodsFor: 'visual properties' stamp: 'MR 2/2/2013 16:15:44.345'!
highlightLine: anInteger
	| line |

	self submorphs do:
		[:morph | morph 
					backgroundColor: Color white; 
					textColor: Color black].
	line := self submorphs at: anInteger.
	line 
		backgroundColor: Color green; 
		textColor: Color white.! !

!SudokuStrategyDisplay methodsFor: 'initialization' stamp: 'MR 1/12/2013 15:55:26.043'!
initialize

	super initialize.
	textLines := OrderedCollection new.
	self 
		layoutPolicy: TableLayout new;
		listDirection: #topToBottom;
		hResizing: #rigid;
		vResizing: #spaceFill;
		width: 300;
		wrapCentering: #topLeft;
		layoutInset: 10;
		borderWidth: 3;
		borderColor: Color black;
		color: Color white;
		disableDragNDrop.
		
	
		! !

!SudokuStrategyDisplay methodsFor: 'initialization' stamp: 'MR 1/10/2013 16:59'!
initializeStrategyTexts: anOrderedCollection

	anOrderedCollection do:
		[:strategy| self createLine: (strategy strategyName).].
	
	self addAllMorphs: textLines! !

!SudokuStrategyDisplay methodsFor: 'visual properties' stamp: 'MR 2/2/2013 16:14:00.705'!
stopHighlighting

	self submorphs do:
		[:morph | morph 
						backgroundColor: Color white; 
						textColor: Color black.].! !

IconicButton subclass: #SudokuPlayButton
	instanceVariableNames: 'state playForm pauseForm sudokuSolver'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Sudoku-GUI'!

!SudokuPlayButton methodsFor: 'initialization' stamp: 'MR 1/12/2013 19:59:01.664'!
initialize

	super initialize.
	state := false.
	self stopStepping.! !

!SudokuPlayButton methodsFor: 'accessing' stamp: 'MR 1/12/2013 17:38:44.958'!
pauseForm
	"Answer the value of pauseForm"

	^ pauseForm! !

!SudokuPlayButton methodsFor: 'accessing' stamp: 'MR 1/12/2013 17:39:14.606'!
pauseForm: aForm
	"Set the value of pauseForm"

	pauseForm := aForm! !

!SudokuPlayButton methodsFor: 'accessing' stamp: 'MR 1/12/2013 17:38:44.899'!
playForm
	"Answer the value of playForm"

	^ playForm! !

!SudokuPlayButton methodsFor: 'accessing' stamp: 'MR 1/12/2013 17:42:49.28'!
playForm: aForm
	"Set the value of playForm"

	playForm := aForm.
	self labelGraphic: playForm.! !

!SudokuPlayButton methodsFor: 'stepping and presenter' stamp: 'MR 1/12/2013 20:06:51.194'!
playOrPause

	state 
		ifTrue: [self stopStepping.
				self labelGraphic: playForm]
		ifFalse: [self startStepping.
				self labelGraphic: pauseForm].
	state := state not.! !

!SudokuPlayButton methodsFor: 'accessing' stamp: 'MR 1/12/2013 17:38:44.827'!
state
	"Answer the value of state"

	^ state! !

!SudokuPlayButton methodsFor: 'accessing' stamp: 'MR 1/12/2013 17:39:41.788'!
state: aBoolean
	"Set the value of state"

	state := aBoolean! !

!SudokuPlayButton methodsFor: 'stepping and presenter' stamp: 'MR 2/2/2013 16:08:40.152'!
step

	super step.
	state ifTrue: [sudokuSolver takeStep
					ifFalse: [self playOrPause]].! !

!SudokuPlayButton methodsFor: 'stepping and presenter' stamp: 'mg 1/17/2013 15:35:40.419'!
stepTime
	^ 200! !

!SudokuPlayButton methodsFor: 'accessing' stamp: 'MR 1/12/2013 17:38:45.008'!
sudokuSolver
	"Answer the value of sudokuSolver"

	^ sudokuSolver! !

!SudokuPlayButton methodsFor: 'accessing' stamp: 'MR 1/12/2013 17:39:52.906'!
sudokuSolver: aSudokuSolver
	"Set the value of sudokuSolver"

	sudokuSolver := aSudokuSolver! !

Object subclass: #BacktrackingStrategy
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Sudoku-Strategies'!

!BacktrackingStrategy methodsFor: 'as yet unclassified' stamp: 'mg 2/3/2013 13:48:31.209'!
solveFromCandidates: aMatrix
"The method returns false if the given matrix is not a valid Sudoku grid"
	| inputMatrix |
	(SudokuStrategyRulesChecker finished: aMatrix) ifTrue: [^true].   
	(SudokuStrategyRulesChecker isValid: aMatrix) ifFalse: [^false].   
	inputMatrix := SudokuGrid newFrom: aMatrix. 
	aMatrix withIndicesDo: [:candidates :x :y |
		(candidates size > 1 and: [candidates size < 5])
			ifTrue: ["test all cases"
				candidates do: [:testCandidate | | newGrid attempt rules |
					newGrid := SudokuGrid newFrom: aMatrix.
					(newGrid at: x at: y) removeAll; add: testCandidate.
					rules := SudokuStrategyRulesChecker new. 
					rules solveFromCandidates: newGrid.
					newGrid := rules candidates.
					attempt := BacktrackingStrategy new solveFromCandidates: newGrid.
					attempt 
						ifFalse: [(inputMatrix at: x at: y) remove: testCandidate]
						ifTrue: [^true]].
				^BacktrackingStrategy new solveFromCandidates: inputMatrix]].
	^ true! !

Object subclass: #SudokuBuilder
	instanceVariableNames: 'sudokuGUI sudokuSolver strategies'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Sudoku-GUI'!

!SudokuBuilder class methodsFor: 'building' stamp: 'MR 12/15/2012 17:12'!
createBoxFromMorph: aMorph
	"This method creates a 3x3 box. The submorphs of the box are copies of aMorph."
	| column box |
	
	column := self createTableOfThreeCellsWithDirection: #topToBottom fromMorph: aMorph.
	box := self createTableOfThreeCellsWithDirection: #leftToRight fromMorph: column.
	^ box! !

!SudokuBuilder class methodsFor: 'building' stamp: 'MR 12/15/2012 17:28'!
createTableOfThreeCellsWithDirection: aSymbol fromMorph: aMorph
	"This method creates three cells in a row (aSymbol = #leftToRight) or column (aSymbol = #topToBottom)."
	| table |
	
	table := RectangleMorph new
		layoutPolicy: TableLayout new;
		listDirection: aSymbol;
		hResizing: #shrinkWrap;
		vResizing: #shrinkWrap;
		borderWidth: 0;
		layoutInset: 0;
		color: Color black;
		disableDragNDrop.
	3 timesRepeat: [table addMorph: aMorph copy].
	^ table! !

!SudokuBuilder methodsFor: 'building' stamp: 'mg 2/4/2013 11:28:07.569'!
buildSudoku
	self buildSudoku: 'sudoku.txt'.! !

!SudokuBuilder methodsFor: 'building' stamp: 'mg 2/4/2013 11:27:16.778'!
buildSudoku: fileName
	self 
		buildSudokuSolver;
		buildSudokuGUI;
		buildSudokuSolverStates.
	sudokuSolver 
		fileName: fileName;  
		takeStep; takeStep.
	sudokuGUI openInWorld.! !

!SudokuBuilder methodsFor: 'building' stamp: 'MR 2/2/2013 19:16:15.283'!
buildSudokuGUI
	| strategyDisplay historySlider |
	sudokuGUI := SudokuGUI new.
	sudokuGUI createButtons: sudokuSolver.
	historySlider := sudokuGUI createSlider: sudokuSolver.
	strategyDisplay := sudokuGUI createStrategyDisplay: strategies.
	sudokuGUI connectSubmorphs.
	sudokuSolver strategyDisplay: strategyDisplay.
	sudokuSolver historySlider: historySlider.! !

!SudokuBuilder methodsFor: 'building' stamp: 'mg 2/3/2013 01:12:03.354'!
buildSudokuSolver
	sudokuSolver := SudokuSolver new. 
	strategies := OrderedCollection 
					with: SudokuStrategyRulesChecker new 
					with: SudokuStrategyHiddenSingles new 
					with: SudokuStrategyNakedPairs new
					with: SudokuStrategyHiddenPairs new
					with: SudokuStrategyPointingPairs new
					with: SudokuStrategyBoxLineReduction new.
	strategies add: SudokuStrategyYWing new.
	sudokuSolver strategies: strategies.! !

!SudokuBuilder methodsFor: 'building' stamp: 'MG 12/15/2012 16:37'!
buildSudokuSolverStates
	| allStates |
	allStates := OrderedCollection new.
	allStates 
		add: SudokuStateHighlightCandidates new;
		add: SudokuStateCandidatesToRemove new;
		add: SudokuStateCandidates new;
		add: SudokuStateNumbers new.
		
	allStates do:
		[:state | state sudokuGUI: sudokuGUI; sudokuSolver: sudokuSolver].
	
	sudokuSolver 
		allStates: allStates;
		currentState: 3.
! !

Object subclass: #SudokuSolvedCellsChecker
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Sudoku-Calculation'!

!SudokuSolvedCellsChecker methodsFor: 'solving' stamp: 'MG 11/24/2012 15:40'!
findSolvedCellsOf: aMatrix
"This method finds all lists of candidates that contain only one number."
	| numbers |
	numbers := Matrix new: aMatrix rowCount.
	1 to: aMatrix rowCount do:
		[:i | 1 to: aMatrix columnCount do: 
			[:j | ((aMatrix at: i at: j) size == 1)
				ifTrue: [numbers at: i at: j put: ((aMatrix at: i at: j) first)] ]].
	^ numbers! !

Object subclass: #SudokuSolver
	instanceVariableNames: 'strategies currentStrategy currentState allStates strategyDisplay candidatesHistory historyPosition historySlider fileName'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Sudoku-Calculation'!

!SudokuSolver methodsFor: 'accessing' stamp: 'MG 12/14/2012 14:32'!
allStates: anObject
	"Set the value of allStates"

	allStates := anObject! !

!SudokuSolver methodsFor: 'calculation' stamp: 'MR 2/2/2013 15:59:29.137'!
applyStrategy
	strategies 
		detect: [:strategy | strategy solveFromCandidates: self candidatesFromStrategy] 
		ifFound: [:solvingStrategy | currentStrategy := solvingStrategy.
									Transcript show: currentStrategy description.
									strategyDisplay highlightLine: (strategies indexOf: currentStrategy).
									^true]
		ifNone: [Transcript show: 'No strategy found'.
				strategyDisplay stopHighlighting.
				^false]! !

!SudokuSolver methodsFor: 'calculation' stamp: 'MR 2/2/2013 17:20:34.027'!
applyStrategyForCandidates: anOrderedCollection

	strategies 
		detect: [:strategy | strategy solveFromCandidates: anOrderedCollection] 
		ifFound: [:solvingStrategy | currentStrategy := solvingStrategy.
									Transcript show: currentStrategy description.
									strategyDisplay highlightLine: (strategies indexOf: currentStrategy).
									^true]
		ifNone: [Transcript show: 'No strategy found'.
				strategyDisplay stopHighlighting.
				^false]! !

!SudokuSolver methodsFor: 'calculation' stamp: 'MR 2/2/2013 19:55:04.593'!
calculationInHistoryMode

	self applyStrategyForCandidates: (candidatesHistory at: historyPosition).
	historyPosition := historyPosition + 1.
	historySlider relativeValue: historyPosition! !

!SudokuSolver methodsFor: 'calculation' stamp: 'MR 2/2/2013 20:01:39.001'!
calculationInNormalMode

	self applyStrategy
		ifFalse: [self inform: 'No Strategy found!!'.
				^false]. 
		self storeCandidates.
		historySlider 
			value: 1;
			maxRelativeValue: candidatesHistory size.
	^true! !

!SudokuSolver methodsFor: 'accessing' stamp: 'MR 2/2/2013 18:36:51.906'!
candidates
	"Answer the value of candidates"

	candidatesHistory 
		ifEmpty: [candidatesHistory add: self initialCandidates].
	historyPosition == 0
		ifFalse: [^ candidatesHistory at: historyPosition].
	^ candidatesHistory last! !

!SudokuSolver methodsFor: 'accessing' stamp: 'MR 2/2/2013 16:48:53.124'!
candidatesFromStrategy
	
	currentStrategy 
		ifNotNil: [^ currentStrategy candidates]
		ifNil: [^ self candidates]! !

!SudokuSolver methodsFor: 'accessing' stamp: 'MR 12/22/2012 18:26'!
currentState
	"Answer the value of the current state as a state (not an index)"

	^ allStates at: currentState! !

!SudokuSolver methodsFor: 'accessing' stamp: 'MR 12/22/2012 18:26'!
currentState: anInteger
	"Set the index of currentState"

	currentState := anInteger ! !

!SudokuSolver methodsFor: 'as yet unclassified' stamp: 'mg 2/4/2013 11:26:25.311'!
fileName: aString
"Sets the file name of the Sudoku."
	fileName := aString.! !

!SudokuSolver methodsFor: 'accessing' stamp: 'MG 12/15/2012 15:14'!
highlightCandidates
"This method returns a matrix that contains collections with all candidates that should be highlighted."
	^ currentStrategy highlightCandidates! !

!SudokuSolver methodsFor: 'accessing' stamp: 'MR 2/2/2013 19:02:10.632'!
historySlider: aSudokuSlider
	"Set the historySlider"

	historySlider := aSudokuSlider  ! !

!SudokuSolver methodsFor: 'calculation' stamp: 'MR 2/2/2013 20:00:05.41'!
inHistoryMode 
	"test whether historyPosition is in bounds so that we can use candidatesHistory"
	(historyPosition == 0)
		ifTrue: [^false].
	(historyPosition >= candidatesHistory size)
		ifTrue: [historyPosition := 0. "stop history mode"
				^false].
	^true! !

!SudokuSolver methodsFor: 'initialize-release' stamp: 'mg 2/4/2013 11:29:28.701'!
initialCandidates
	| allCandidates file numberList numbers candidates |

	numberList := OrderedCollection new.
	file := MultiByteFileStream fileNamed: fileName.
	file upToEnd asString substrings
		collect: [:row | row do: [:digit | numberList add: digit digitValue]].

	numbers := Matrix new: 9 tabulate: [:i :j | numberList at: (j \\ 10 + (i-1*9))].
	file close.
	
	allCandidates := OrderedCollection new.
	1 to: 9 do:
		[:i | allCandidates add: i].
	
	candidates := Matrix new: 9.
	1 to: 9 do:
		[:i | 1 to: 9 do:
			[:j | ((numbers at: i at: j) = 0)
					ifTrue: [candidates at: i at: j put: allCandidates copy]
					ifFalse: [candidates at: i at: j put: (OrderedCollection with: (numbers at: i at: j))]]].
	^ candidates! !

!SudokuSolver methodsFor: 'initialization' stamp: 'MR 2/2/2013 18:37:34.871'!
initialize

	super initialize.
	
	candidatesHistory := OrderedCollection new.
	historyPosition := 0. ! !

!SudokuSolver methodsFor: 'accessing' stamp: 'MG 12/15/2012 15:48'!
numbers
	^ SudokuSolvedCellsChecker new findSolvedCellsOf: self candidates.! !

!SudokuSolver methodsFor: 'accessing' stamp: 'MG 12/15/2012 16:17'!
removeCandidates
"This method returns a matrix that contains collections with all candidates that should be highlighted."
	^ currentStrategy removeCandidates! !

!SudokuSolver methodsFor: 'accessing' stamp: 'MR 2/2/2013 17:39:32.703'!
sizeOfHistory

	^ candidatesHistory size! !

!SudokuSolver methodsFor: 'calculation' stamp: 'MR 2/2/2013 16:59:07.929'!
storeCandidates 
	
	candidatesHistory add: self candidatesFromStrategy.! !

!SudokuSolver methodsFor: 'accessing' stamp: 'MR 1/10/2013 15:02'!
strategies: anOrderedCollection
	"Set the strategies"

	strategies := anOrderedCollection ! !

!SudokuSolver methodsFor: 'accessing' stamp: 'mg 1/10/2013 21:15:48.455'!
strategyDisplay: aStrategyDisplay
	"Set the strageyDisplay"

	strategyDisplay := aStrategyDisplay ! !

!SudokuSolver methodsFor: 'calculation' stamp: 'MR 2/2/2013 20:02:34.133'!
takeStep

	(currentState = 1)
		ifTrue: [self inHistoryMode
					ifTrue: [self calculationInHistoryMode]
					ifFalse: [self calculationInNormalMode 
								ifFalse: [^false]]].
	self updateState.
	^true! !

!SudokuSolver methodsFor: 'calculation' stamp: 'MR 2/2/2013 18:35:07.622'!
takeStepFromHistoryAt: anInteger

	currentState := 3.
	historyPosition := anInteger.
	self 
		updateState; "set candidates"
		updateState. "set numbers"
	^true! !

!SudokuSolver methodsFor: 'calculation' stamp: 'MR 2/2/2013 16:19:14.346'!
updateState

	self currentState updateSudokuGUI.
	currentState := currentState \\ allStates size + 1. 
	self currentState setButtonName.! !

Object subclass: #SudokuSolverState
	instanceVariableNames: 'emptyCell sudokuSolver sudokuGUI'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Sudoku-Calculation'!

!SudokuSolverState methodsFor: 'naming' stamp: 'MR 12/22/2012 17:18'!
description 
	"explanation of what will be done in this state"
	self subclassResponsibility! !

!SudokuSolverState methodsFor: 'initialize-release' stamp: 'MG 12/14/2012 15:23'!
initialize
	super initialize.
	emptyCell := nil.! !

!SudokuSolverState methodsFor: 'updating' stamp: 'MR 1/10/2013 15:29'!
setButtonName 

	sudokuGUI setButtonName: self stateName.! !

!SudokuSolverState methodsFor: 'updating' stamp: 'MR 2/2/2013 18:32:43.703'!
setSudokuGUI
	"some states do not support this method"! !

!SudokuSolverState methodsFor: 'naming' stamp: 'MR 1/10/2013 15:28'!
stateName
	"name of the state which should be shown to the user"
	self subclassResponsibility ! !

!SudokuSolverState methodsFor: 'accessing' stamp: 'MG 12/14/2012 14:34'!
sudokuGUI: anObject
	"Set the value of sudokuGUI"

	sudokuGUI := anObject! !

!SudokuSolverState methodsFor: 'accessing' stamp: 'MG 12/14/2012 15:14'!
sudokuSolver: anObject
	"Set the value of sudokuSolver"

	sudokuSolver := anObject! !

!SudokuSolverState methodsFor: 'updating' stamp: 'MG 12/14/2012 14:38'!
updateSudokuGUI
	self subclassResponsibility.! !

SudokuSolverState subclass: #SudokuStateCandidates
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Sudoku-Calculation'!

!SudokuStateCandidates methodsFor: 'naming' stamp: 'MR 12/22/2012 17:42'!
description 

	^ 'remove candidates which were found as being removable'! !

!SudokuStateCandidates methodsFor: 'updating' stamp: 'MR 2/2/2013 18:32:12.533'!
setSudokuGUI
	sudokuGUI setCells: sudokuSolver candidates ofType: #candidates.! !

!SudokuStateCandidates methodsFor: 'naming' stamp: 'MR 1/10/2013 15:29'!
stateName

	^ 'Delete Removable Candidates'! !

!SudokuStateCandidates methodsFor: 'updating' stamp: 'MR 2/2/2013 16:50:07.444'!
updateSudokuGUI
	sudokuGUI updateCells: sudokuSolver candidates ofType: #candidates.! !

SudokuSolverState subclass: #SudokuStateCandidatesToRemove
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Sudoku-Calculation'!

!SudokuStateCandidatesToRemove methodsFor: 'naming' stamp: 'MR 12/22/2012 17:42'!
description 

	^ 'show candidates which can be removed in the next step'! !

!SudokuStateCandidatesToRemove methodsFor: 'naming' stamp: 'MR 1/10/2013 15:29'!
stateName

	^ 'Show Removable Candidates'! !

!SudokuStateCandidatesToRemove methodsFor: 'updating' stamp: 'MG 12/15/2012 16:48'!
updateSudokuGUI
	| removeCandidates |
	removeCandidates := sudokuSolver removeCandidates.
	sudokuGUI updateCells: removeCandidates ofType: #removeCandidates.! !

SudokuSolverState subclass: #SudokuStateHighlightCandidates
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Sudoku-Calculation'!

!SudokuStateHighlightCandidates methodsFor: 'naming' stamp: 'MR 12/22/2012 17:43'!
description 

	^ 'highlight candidates which lead to removement of other candidates'! !

!SudokuStateHighlightCandidates methodsFor: 'naming' stamp: 'MR 1/10/2013 15:29'!
stateName

	^ 'Highlight Important Candidates'! !

!SudokuStateHighlightCandidates methodsFor: 'updating' stamp: 'MG 12/15/2012 15:12'!
updateSudokuGUI
	| highlightCandidates |
	highlightCandidates := sudokuSolver highlightCandidates.
	sudokuGUI updateCells: highlightCandidates ofType: #highlight.! !

SudokuSolverState subclass: #SudokuStateNumbers
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Sudoku-Calculation'!

!SudokuStateNumbers methodsFor: 'naming' stamp: 'MR 12/22/2012 17:43'!
description 

	^ 'transform fields with just one candidate to a number'! !

!SudokuStateNumbers methodsFor: 'updating' stamp: 'MR 2/2/2013 18:34:33.366'!
setSudokuGUI
	| numbers numbersAsString |
	
	numbers := sudokuSolver numbers.
	numbersAsString := Matrix new: 9.
	1 to: 9 do:
		[:i | 1 to: 9 do:
			[:j | ((numbers at: i at: j) == emptyCell)
					ifTrue: [numbersAsString at: i at: j put: '']
					ifFalse: [numbersAsString at: i at: j put: (numbers at: i at: j) asString] ]].
	
	sudokuGUI setCells: numbersAsString ofType: #number.! !

!SudokuStateNumbers methodsFor: 'naming' stamp: 'MR 1/10/2013 15:29'!
stateName

	^ 'Show Solved Cells'! !

!SudokuStateNumbers methodsFor: 'updating' stamp: 'MG 12/15/2012 15:11'!
updateSudokuGUI
	| numbers numbersAsString |
	
	numbers := sudokuSolver numbers.
	numbersAsString := Matrix new: 9.
	1 to: 9 do:
		[:i | 1 to: 9 do:
			[:j | ((numbers at: i at: j) == emptyCell)
					ifTrue: [numbersAsString at: i at: j put: '']
					ifFalse: [numbersAsString at: i at: j put: (numbers at: i at: j) asString] ]].
	
	sudokuGUI updateCells: numbersAsString ofType: #number.! !

Object subclass: #SudokuStrategy
	instanceVariableNames: 'solvedCells inputMatrix candidates highlightCandidates removeCandidates'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Sudoku-Strategies'!

!SudokuStrategy methodsFor: 'accessing' stamp: 'MR 12/15/2012 15:04'!
candidates
	"Answer the value of candidates"

	^ candidates! !

!SudokuStrategy methodsFor: 'accessing' stamp: 'MR 12/15/2012 15:04'!
candidates: anObject
	"Set the value of candidates"

	candidates := anObject! !

!SudokuStrategy methodsFor: 'solving' stamp: 'mg 12/19/2012 13:42:30.046'!
checkStrategyForBoxes
	self subclassResponsibility.! !

!SudokuStrategy methodsFor: 'solving' stamp: 'mg 12/19/2012 13:42:14.894'!
checkStrategyForColumns
	self subclassResponsibility.! !

!SudokuStrategy methodsFor: 'solving' stamp: 'mg 12/19/2012 13:42:01.926'!
checkStrategyForRows
	self subclassResponsibility.! !

!SudokuStrategy methodsFor: 'naming' stamp: 'mg 12/19/2012 14:48:17.668'!
description
	self subclassResponsibility.! !

!SudokuStrategy methodsFor: 'solving' stamp: 'mg 2/2/2013 15:50:55.337'!
findHighlightCandidates! !

!SudokuStrategy methodsFor: 'solving' stamp: 'mg 1/10/2013 22:29:41.24'!
findRemoveCandidates

	removeCandidates := SudokuGrid newFrom: inputMatrix.
	removeCandidates indicesDo:
		[:i :j | (removeCandidates at: i at: j) removeAll: (candidates at: i at:j)]! !

!SudokuStrategy methodsFor: 'accessing' stamp: 'mg 12/19/2012 13:46:13.439'!
getBoxCellsAt: i at: j in: aMatrix
	| cells |
	cells := OrderedCollection new.
	0 to: 2 do:
		[:x | 0 to: 2 do:
			[:y | cells add: (aMatrix at: i+x at: j+y)]].
	^ cells! !

!SudokuStrategy methodsFor: 'accessing' stamp: 'MR 12/15/2012 15:54'!
highlightCandidates
	"Answer the value of highlightCandidates"

	^ highlightCandidates! !

!SudokuStrategy methodsFor: 'accessing' stamp: 'MR 12/15/2012 15:55'!
highlightCandidates: anObject
	"Set the value of highlightCandidates"

	highlightCandidates := anObject! !

!SudokuStrategy methodsFor: 'as yet unclassified' stamp: 'mg 2/1/2013 14:34:32.987'!
indecesPerNumberIn: aCollection
"This method returns a dictionary with all possible numbers as keys. The values are lists of all indices in aCollection, where the number is included. The given collection can be a row of the candidates matrix."
	| dict |
	dict := Dictionary new.
	1 to: aCollection size do: [:number |
		dict at: number put: OrderedCollection new].
	aCollection withIndexDo: [:candidateList :index |
		candidateList do: [:number |
			(dict at: number) add: index]].
	^ dict! !

!SudokuStrategy methodsFor: 'solving' stamp: 'mg 1/10/2013 22:53:53.085'!
initializeHighlightCandidates

	highlightCandidates := SudokuGrid new: 9.
	highlightCandidates indicesDo:   
		[:i :j | highlightCandidates at: i at: j put: OrderedCollection new].! !

!SudokuStrategy methodsFor: 'accessing' stamp: 'MG 12/15/2012 16:55'!
inputMatrix
	"Answer the value of inputMatrix"

	^ inputMatrix! !

!SudokuStrategy methodsFor: 'accessing' stamp: 'MG 12/15/2012 16:55'!
inputMatrix: anObject
	"Set the value of inputMatrix"

	inputMatrix := anObject! !

!SudokuStrategy methodsFor: 'accessing' stamp: 'MR 12/15/2012 15:55'!
removeCandidates
	"Answer the value of removeCandidates"

	^ removeCandidates! !

!SudokuStrategy methodsFor: 'accessing' stamp: 'MR 12/15/2012 15:56'!
removeCandidates: anObject
	"Set the value of removeCandidates"

	removeCandidates := anObject! !

!SudokuStrategy methodsFor: 'solving' stamp: 'mg 1/10/2013 22:41:22.205'!
solveFromCandidates: aMatrix
	inputMatrix := SudokuGrid newFrom: aMatrix.
	candidates := SudokuGrid newFrom: aMatrix.
	solvedCells := false.
	self
		initializeHighlightCandidates;
		checkStrategyForRows;
		checkStrategyForColumns;
		checkStrategyForBoxes;
		findRemoveCandidates;
		findHighlightCandidates.
	^ solvedCells! !

!SudokuStrategy methodsFor: 'accessing' stamp: 'MG 12/15/2012 16:55'!
solvedCells
	"Answer the value of solvedCells"

	^ solvedCells! !

!SudokuStrategy methodsFor: 'accessing' stamp: 'MG 12/15/2012 16:55'!
solvedCells: anObject
	"Set the value of solvedCells"

	solvedCells := anObject! !

!SudokuStrategy methodsFor: 'naming' stamp: 'MR 1/10/2013 15:25'!
strategyName
	self subclassResponsibility.! !

SudokuStrategy subclass: #SudokuStrategyBoxLineReduction
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Sudoku-Strategies'!

!SudokuStrategyBoxLineReduction methodsFor: 'solving' stamp: 'mg 2/2/2013 15:53:04.256'!
checkStrategyForBoxes
"The strategy for boxes is called PointingPairs"
! !

!SudokuStrategyBoxLineReduction methodsFor: 'solving' stamp: 'mg 2/2/2013 17:10:12.093'!
checkStrategyForColumns
"This method searchs for numbers that can only be in one box, if we look at a column"
	1 to: inputMatrix columnCount do:
		[:column | 	| numberIndex numbersForReduction |
					numberIndex := self indecesPerNumberIn: (inputMatrix atColumn: column).  
					numbersForReduction := numberIndex select: 
						[:each | each size > 1 and: [each allSatisfy: [:number | (number-1 // 3) = (each first - 1 // 3)]]].
					numbersForReduction associationsDo: [:each |  | highlight box indices |
						highlight := false.
						indices := OrderedCollection new.
						indices add: (SudokuGrid boxIndexFromCellIndex: each value first @ column).
						indices add: (SudokuGrid boxIndexFromCellIndex: each value second @ column).
						each value size = 3 ifTrue: [indices add: (SudokuGrid boxIndexFromCellIndex: each value third @ column)]. 
					
						box := candidates atBox: (SudokuGrid boxNumberFromIndex: each value first @ column).
						box withIndexDo: [:candidateList :index | 
							(indices includes: index)
								ifFalse: [
									(candidateList includes: each key) ifTrue: [solvedCells := true. highlight := true].
									candidateList remove: each key ifAbsent: []	]].
						
						highlight ifTrue: [each value do: [:index |((highlightCandidates atColumn: column) at: index) add: each key]]]]! !

!SudokuStrategyBoxLineReduction methodsFor: 'solving' stamp: 'mg 2/2/2013 17:00:42.701'!
checkStrategyForRows
"This method searchs for numbers that can only be in one box, if we look at a row"
	1 to: inputMatrix rowCount do:
		[:row | 	| numberIndex numbersForReduction |
				numberIndex := self indecesPerNumberIn: (inputMatrix atRow: row).  
				numbersForReduction := numberIndex select: 
					[:each | each size > 1 and: [each allSatisfy: [:number | (number-1 // 3) = (each first - 1 // 3)]]].
				numbersForReduction associationsDo: [:each |  | highlight box indices |
					highlight := false.
					indices := OrderedCollection new.
					indices add: (SudokuGrid boxIndexFromCellIndex: row @ each value first).
					indices add: (SudokuGrid boxIndexFromCellIndex: row @ each value second).
					each value size = 3 ifTrue: [indices add: (SudokuGrid boxIndexFromCellIndex: row @ each value third)]. 
					
					box := candidates atBox: (SudokuGrid boxNumberFromIndex: row @ each value first).
					box withIndexDo: [:candidateList :index | 
						(indices includes: index)
							ifFalse: [
								(candidateList includes: each key) ifTrue: [solvedCells := true. highlight := true].
								candidateList remove: each key ifAbsent: []	]].
						
					highlight ifTrue: [each value do: [:index |((highlightCandidates atRow: row) at: index) add: each key]]].
					]! !

!SudokuStrategyBoxLineReduction methodsFor: 'naming' stamp: 'mg 2/2/2013 15:53:27.265'!
description
	^'Box Line Reduction'! !

!SudokuStrategyBoxLineReduction methodsFor: 'naming' stamp: 'mg 2/2/2013 15:53:33.225'!
strategyName
	^'Box Line Reduction'! !

SudokuStrategy subclass: #SudokuStrategyHiddenPairs
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Sudoku-Strategies'!

!SudokuStrategyHiddenPairs methodsFor: 'solving' stamp: 'mg 2/2/2013 11:57:57.802'!
checkStrategyForBoxes
"This method searches for pairs that have only two possible cells in a box."
	1 to: inputMatrix boxCount do:
		[:box| | numberIndex pairsDict |
				numberIndex := self indecesPerNumberIn: (inputMatrix atBox: box).  
				pairsDict := numberIndex select: [:each | each size = 2 and: [(numberIndex values occurrencesOf: each) > 1]].
				pairsDict size > 2 ifTrue: [^self]. "not yet implemented"
				pairsDict ifNotEmpty: [ | highlight |
					highlight := false.
					pairsDict values first do: [:index |
						((candidates atBox: box) at: index) size > 2
							ifTrue: [solvedCells := true. highlight := true]  .
						((candidates atBox: box) at: index) removeAllSuchThat: [:number | (pairsDict keys includes: number) not]].
					highlight ifTrue: [
							((highlightCandidates atBox: box) at: (pairsDict values first at: 1)) addAll: pairsDict keys.
							((highlightCandidates atBox: box) at: (pairsDict values first at: 2)) addAll: pairsDict keys]]].! !

!SudokuStrategyHiddenPairs methodsFor: 'solving' stamp: 'mg 2/2/2013 11:57:42.441'!
checkStrategyForColumns
"This method searches for pairs that have only two possible cells in a column."
	1 to: inputMatrix columnCount do:
		[:column| | numberIndex pairsDict |
				numberIndex := self indecesPerNumberIn: (inputMatrix atColumn: column).  
				pairsDict := numberIndex select: [:each | each size = 2 and: [(numberIndex values occurrencesOf: each) > 1]].
				pairsDict size > 2 ifTrue: [^self]. "not yet implemented"
				pairsDict ifNotEmpty: [ | highlight |
					highlight := false.
					pairsDict values first do: [:index |
						((candidates atColumn: column) at: index) size > 2
							ifTrue: [solvedCells := true. highlight := true]  .
						((candidates atColumn: column) at: index) removeAllSuchThat: [:number | (pairsDict keys includes: number) not]].
					highlight ifTrue: [
							(highlightCandidates at: (pairsDict values first at: 1) at: column) addAll: pairsDict keys.
							(highlightCandidates at: (pairsDict values first at: 2) at: column) addAll: pairsDict keys]]].! !

!SudokuStrategyHiddenPairs methodsFor: 'solving' stamp: 'mg 2/2/2013 11:57:51.17'!
checkStrategyForRows
"This method searches for pairs that have only two possible cells in a row."
	1 to: inputMatrix rowCount do:
		[:row | 	| numberIndex pairsDict |
				numberIndex := self indecesPerNumberIn: (inputMatrix atRow: row).  
				pairsDict := numberIndex select: [:each | each size = 2 and: [(numberIndex values occurrencesOf: each) > 1]].
				pairsDict size > 2 ifTrue: [^self]. "not yet implemented"
				pairsDict ifNotEmpty: [ | highlight |
						highlight := false.
						pairsDict values first do: [:index |
						((candidates atRow: row) at: index) size > 2
							ifTrue: [solvedCells := true. highlight := true].
						(candidates at: row at: index) removeAllSuchThat: [:number | (pairsDict keys includes: number) not]].
						highlight ifTrue: [
							(highlightCandidates at: row at: (pairsDict values first at: 1)) addAll: pairsDict keys.
							(highlightCandidates at: row at: (pairsDict values first at: 2)) addAll: pairsDict keys]]].! !

!SudokuStrategyHiddenPairs methodsFor: 'naming' stamp: 'mg 12/19/2012 14:58:43.844'!
description
	^ 'Hidden Pairs'! !

!SudokuStrategyHiddenPairs methodsFor: 'naming' stamp: 'MR 1/10/2013 15:27'!
strategyName
	^ 'Hidden Pairs'! !

SudokuStrategy subclass: #SudokuStrategyHiddenSingles
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Sudoku-Strategies'!

!SudokuStrategyHiddenSingles methodsFor: 'solving' stamp: 'mg 1/10/2013 14:35:23.891'!
checkStrategyForBoxes
"This method searches for numbers that have only one possible cell in a box. (i,j) are the indices of the top left cells in each box."
	1 to: 9 by: 3 do:
		[:i | 1 to: 9 by: 3 do:
			[:j | 1 to: 9 do:
				[:number | 	| index |  	
							index := self number: number hasOnePossibilityIn: (self getBoxCellsAt: i at: j in: inputMatrix ).
							(index = 0)
								ifFalse: [self removeCandidatesInCell: ((self getBoxCellsAt: i at: j in: candidates) at: index) 
											hiddenSingle: number.
										(highlightCandidates at: i+(index-1//3) at: j+(index-1\\3))add: number]]]].! !

!SudokuStrategyHiddenSingles methodsFor: 'solving' stamp: 'mg 1/10/2013 14:38:50.139'!
checkStrategyForColumns
"This method searches for numbers that have only one possible cell in a column."
	1 to: candidates columnCount do:
		[:column | 1 to: 9 do: 
			[:number |	| index | 	
						index := self number: number hasOnePossibilityIn: (inputMatrix atColumn: column).
						(index = 0)
							ifFalse: [self removeCandidatesInCell: ((candidates atColumn: column) at: index) 
										hiddenSingle: number. 
									(highlightCandidates at: index at: column) add: number]]].! !

!SudokuStrategyHiddenSingles methodsFor: 'solving' stamp: 'mg 1/10/2013 14:28:35.157'!
checkStrategyForRows
"This method searches for numbers that have only one possible cell in a row."
	1 to: candidates rowCount do:
		[:row | 1 to: 9 do: 
			[:number | 	| index | 	
						index := self number: number hasOnePossibilityIn: (inputMatrix atRow: row).
						(index = 0)
							ifFalse: [self removeCandidatesInCell: ((candidates atRow: row) at: index) 
										hiddenSingle: number. 
									(highlightCandidates at: row at: index) add: number]]].! !

!SudokuStrategyHiddenSingles methodsFor: 'naming' stamp: 'mg 12/19/2012 14:47:08.384'!
description
	^ 'Hidden Singles'! !

!SudokuStrategyHiddenSingles methodsFor: 'solving' stamp: 'mg 12/19/2012 14:00:37.973'!
number: aNumber hasOnePossibilityIn: aCollection
"This method checks whether aNumber can only be in one cell of aCollection. If that is the case, the index of that cell will be returned. Otherwise, this method returns 0."
	| numberOfPossibleCells indexOfCell |
	numberOfPossibleCells := 0.
	indexOfCell := 0.
	aCollection withIndexDo: 
		[:candidates :index | (candidates size = 1 and: [candidates includes: aNumber])
								ifTrue: [^ 0].  
							(candidates includes: aNumber)
								ifTrue: [	numberOfPossibleCells := numberOfPossibleCells + 1. 
										indexOfCell := index]].
					
	(numberOfPossibleCells = 1)
		ifFalse: [indexOfCell := 0].
		
	^ indexOfCell! !

!SudokuStrategyHiddenSingles methodsFor: 'solving' stamp: 'mg 12/19/2012 14:01:01.119'!
removeCandidatesInCell: aCollection hiddenSingle: aNumber
	solvedCells := solvedCells or: [aCollection size > 1].
	aCollection removeAllSuchThat: [:number | (number = aNumber) not].! !

!SudokuStrategyHiddenSingles methodsFor: 'naming' stamp: 'MR 1/10/2013 15:26'!
strategyName
	^ 'Hidden Singles'! !

SudokuStrategy subclass: #SudokuStrategyNakedPairs
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Sudoku-Strategies'!

!SudokuStrategyNakedPairs methodsFor: 'solving' stamp: 'MG 12/19/2012 12:13'!
checkStrategyForBoxAt: i at: j
	| box listOfPairs possiblePairs |
	box := self getBoxCellsAt: i at: j in: inputMatrix. 
	possiblePairs := self findCellsWithTwoPossibilitiesIn: box.
	listOfPairs := self searchForPairs: possiblePairs.
	listOfPairs notEmpty
		ifTrue: [self removeCandidatesForPairs: listOfPairs in: (self getBoxCellsAt: i at: j in: candidates)].! !

!SudokuStrategyNakedPairs methodsFor: 'solving' stamp: 'MG 12/19/2012 12:06'!
checkStrategyForBoxes
"(i,j) are the indices of the top left cells in each box."
	1 to: 9 by: 3 do:
		[:i | 1 to: 9 by: 3 do:
			[:j | self checkStrategyForBoxAt: i at: j]].! !

!SudokuStrategyNakedPairs methodsFor: 'solving' stamp: 'MG 12/19/2012 00:58'!
checkStrategyForColumns
	1 to: inputMatrix columnCount do:
		[:column | | possibleCells listOfPairs | 
				possibleCells := self findCellsWithTwoPossibilitiesIn: (inputMatrix atColumn: column).
				listOfPairs := self searchForPairs: possibleCells.
				listOfPairs notEmpty
					ifTrue: [self removeCandidatesForPairs: listOfPairs in: (candidates atColumn: column)]]! !

!SudokuStrategyNakedPairs methodsFor: 'solving' stamp: 'MG 12/19/2012 00:29'!
checkStrategyForRows
	1 to: inputMatrix rowCount do:
		[:row | | possibleCells listOfPairs | 
				possibleCells := self findCellsWithTwoPossibilitiesIn: (inputMatrix atRow: row).
				listOfPairs := self searchForPairs: possibleCells.
				listOfPairs notEmpty
					ifTrue: [self removeCandidatesForPairs: listOfPairs in: (candidates atRow: row)]]! !

!SudokuStrategyNakedPairs methodsFor: 'naming' stamp: 'mg 12/19/2012 14:46:54.76'!
description
	^ 'Naked Pairs'! !

!SudokuStrategyNakedPairs methodsFor: 'solving' stamp: 'MG 12/19/2012 00:29'!
findCellsWithTwoPossibilitiesIn: aCollection
	^ aCollection select: [:candidates | candidates size = 2].! !

!SudokuStrategyNakedPairs methodsFor: 'solving' stamp: 'mg 1/17/2013 16:06:00.943'!
findHighlightCandidates
	self 
		findHighlightCandidatesForRows;
		findHighlightCandidatesForColumns;
		findHighlightCandidatesForBoxes.! !

!SudokuStrategyNakedPairs methodsFor: 'solving' stamp: 'mg 1/17/2013 16:05:52.534'!
findHighlightCandidatesForBoxes
	1 to: inputMatrix boxCount do:
		[:boxIndex | | pairs | 
				pairs := OrderedCollection new.
				1 to: inputMatrix boxCount - 1 do:
					[:cellIndex | | candidateList cell |
							cell := SudokuGrid indexOfbox: boxIndex cell: cellIndex.
							candidateList := inputMatrix at: cell x at: cell y.
							candidateList size = 2
								ifTrue: [	(cellIndex+1 to: inputMatrix boxCount) 
											do: [:cellIndex2 | | cell2 |
															cell2 := SudokuGrid indexOfbox: boxIndex cell: cellIndex2.
															(candidateList includesAllOf: (inputMatrix at: cell2 x at: cell2 y))
																ifTrue: [pairs add: (OrderedCollection with: cellIndex with: cellIndex2)]]]].
				pairs do: 
					[:pairPositions | | pair | 
									pair := (inputMatrix atBox: boxIndex) at: (pairPositions at: 1).
									((removeCandidates atBox: boxIndex) contains: [:candidateList | candidateList includesAnyOf: pair])
										ifTrue: [ | firstPairIndex secondPairIndex |
												firstPairIndex := SudokuGrid indexOfbox: boxIndex cell: (pairPositions at: 1).
												secondPairIndex := SudokuGrid indexOfbox: boxIndex cell: (pairPositions at: 2).
												(highlightCandidates at: firstPairIndex x at: firstPairIndex y) add: (pair at: 1).
												(highlightCandidates at: firstPairIndex x at: firstPairIndex y) add: (pair at: 2).
												(highlightCandidates at: secondPairIndex x at: secondPairIndex y) add: (pair at: 1).
												(highlightCandidates at: secondPairIndex x at: secondPairIndex y) add: (pair at: 2)]]].! !

!SudokuStrategyNakedPairs methodsFor: 'solving' stamp: 'mg 1/17/2013 15:51:24.737'!
findHighlightCandidatesForColumns
	1 to: inputMatrix columnCount do:
		[:column | | pairs | 
				pairs := OrderedCollection new.
				1 to: inputMatrix rowCount - 1 do:
					[:row | | candidateList |
							candidateList := inputMatrix at: row at: column.
							candidateList size = 2
								ifTrue: [	(row+1 to: inputMatrix rowCount) 
											do: [:row2 | (candidateList includesAllOf: (inputMatrix at: row2 at: column))
																ifTrue: [pairs add: (OrderedCollection with: row with: row2)]]]].
				pairs do: 
					[:pairPositions | | pair | 
									pair := inputMatrix at: (pairPositions at: 1) at: column.
									((removeCandidates atColumn: column) contains: [:candidateList | candidateList includesAnyOf: pair])
										ifTrue: [(highlightCandidates at: (pairPositions at: 1) at: column) add: (pair at: 1).
												(highlightCandidates at: (pairPositions at: 1) at: column) add: (pair at: 2).
												(highlightCandidates at: (pairPositions at: 2) at: column) add: (pair at: 1).
												(highlightCandidates at: (pairPositions at: 2) at: column) add: (pair at: 2)]]].! !

!SudokuStrategyNakedPairs methodsFor: 'solving' stamp: 'mg 1/17/2013 15:45:06.721'!
findHighlightCandidatesForRows
	1 to: inputMatrix rowCount do:
		[:row | | pairs | 
				pairs := OrderedCollection new.
				1 to: inputMatrix columnCount - 1 do:
					[:column | | candidateList |
							candidateList := inputMatrix at: row at: column.
							candidateList size = 2
								ifTrue: [	(column+1 to: inputMatrix columnCount) 
											do: [:column2 | (candidateList includesAllOf: (inputMatrix at: row at: column2))
																ifTrue: [pairs add: (OrderedCollection with: column with: column2)]]]].
				pairs do: 
					[:pairPositions | | pair | 
									pair := inputMatrix at: row at: (pairPositions at: 1).
									((removeCandidates atRow: row) contains: [:candidateList | candidateList includesAnyOf: pair])
										ifTrue: [(highlightCandidates at: row at: (pairPositions at: 1)) add: (pair at: 1).
												(highlightCandidates at: row at: (pairPositions at: 1)) add: (pair at: 2).
												(highlightCandidates at: row at: (pairPositions at: 2)) add: (pair at: 1).
												(highlightCandidates at: row at: (pairPositions at: 2)) add: (pair at: 2)]]].! !

!SudokuStrategyNakedPairs methodsFor: 'solving' stamp: 'MG 12/19/2012 00:50'!
removeCandidatesForPairs: aCollectionOfPairs in: aCollectionOfCandidateLists
	aCollectionOfPairs do:
		[:pair | aCollectionOfCandidateLists do:
			[:candidates | (pair asSet = candidates asSet)
								ifFalse: [(candidates includesAnyOf: pair)
											ifTrue: [solvedCells := true.
													candidates removeAllFoundIn: pair]]]].! !

!SudokuStrategyNakedPairs methodsFor: 'solving' stamp: 'MG 12/19/2012 00:54'!
searchForPairs: aCollection
"This method returns a collection of all pairs in aCollection. A pair is found if two lists have the same candidates."
	| possiblePairs pairs |
	pairs := OrderedCollection new.
	possiblePairs := OrderedCollection new.
	aCollection do:
		[:candidates | |foundPair|
					foundPair := false.
					possiblePairs do:
						[:pair | (pair includesAllOf: candidates) 
									ifTrue: [pairs add: candidates. foundPair := true]].
					foundPair ifFalse: [possiblePairs add: candidates]].
	^ pairs! !

!SudokuStrategyNakedPairs methodsFor: 'naming' stamp: 'MR 1/10/2013 15:26'!
strategyName
	^ 'Naked Pairs'! !

SudokuStrategy subclass: #SudokuStrategyPointingPairs
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Sudoku-Strategies'!

!SudokuStrategyPointingPairs methodsFor: 'solving' stamp: 'mg 2/1/2013 18:11:43.867'!
checkStrategyForBoxes
"This method searchs for numbers that can only be in one row or column, if we look at one box"
	1 to: inputMatrix boxCount do: 
		[:box | 	| numberIndex pointingNumbersRow pointingNumbersColumn |
				numberIndex := self indecesPerNumberIn: (inputMatrix atBox: box).  
				pointingNumbersRow := numberIndex select: 
					[:each | each size > 1 and: [each allSatisfy: [:number | (number-1 // 3) = (each first - 1 // 3)]]].
				pointingNumbersColumn :=  numberIndex select: 
					[:each | each size > 1 and: [each allSatisfy: [:number | (number-1 \\ 3) = (each first - 1 \\ 3)]]].
				
				pointingNumbersRow associationsDo: [:each | | indices row highlight |
					highlight := false.
					indices := OrderedCollection new.
					indices add: (SudokuGrid indexOfbox: box cell: each value first) y.
					indices add: (SudokuGrid indexOfbox: box cell: each value second) y.
					each value size = 3 ifTrue: [indices add: (SudokuGrid indexOfbox: box cell: each value third) y].
					
					row := candidates atRow: (SudokuGrid indexOfbox: box cell: each value first) x.
					row withIndexDo: [:candidatesList :index |
						 (indices includes: index) ifFalse: [
							(candidatesList includes: each key) ifTrue: [solvedCells := true. highlight := true].  
							candidatesList remove: each key ifAbsent: []]].
					
					highlight ifTrue: [each value do: [:index |((highlightCandidates atBox: box) at: index) add: each key]]].
				
				pointingNumbersColumn associationsDo: [:each | | indices column highlight |
					highlight := false.
					indices := OrderedCollection new.
					indices add: (SudokuGrid indexOfbox: box cell: each value first) x.
					indices add: (SudokuGrid indexOfbox: box cell: each value second) x.
					each value size = 3 ifTrue: [indices add: (SudokuGrid indexOfbox: box cell: each value third) x].
					
					column := candidates atColumn: (SudokuGrid indexOfbox: box cell: each value first) y.
					column withIndexDo: [:candidatesList :index |
						 (indices includes: index) ifFalse: [
							(candidatesList includes: each key) ifTrue: [solvedCells := true. highlight := true].  
							candidatesList remove: each key ifAbsent: []]].
					
					highlight ifTrue: [each value do: [:index |((highlightCandidates atBox: box) at: index) add: each key]]].
		]! !

!SudokuStrategyPointingPairs methodsFor: 'solving' stamp: 'mg 2/1/2013 17:25:53.592'!
checkStrategyForColumns
"The strategy for columns is called BoxLineReduction"! !

!SudokuStrategyPointingPairs methodsFor: 'solving' stamp: 'mg 2/1/2013 17:26:04.153'!
checkStrategyForRows
"The strategy for rows is called BoxLineReduction"! !

!SudokuStrategyPointingPairs methodsFor: 'naming' stamp: 'mg 2/1/2013 17:28:24.865'!
description
	^'Pointing Pairs'! !

!SudokuStrategyPointingPairs methodsFor: 'naming' stamp: 'mg 2/1/2013 17:28:31.578'!
strategyName 
	^'Pointing Pairs'! !

SudokuStrategy subclass: #SudokuStrategyRulesChecker
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Sudoku-Strategies'!

!SudokuStrategyRulesChecker class methodsFor: 'as yet unclassified' stamp: 'mg 2/3/2013 12:11:55.052'!
finished: aSudokuGrid
"This method checks if the Sudoku grid is solved completely and correctly."
	(aSudokuGrid allSatisfy: [:candidateList | candidateList size = 1])  
		ifFalse: [^false]
		ifTrue: [^self isValid: aSudokuGrid]    ! !

!SudokuStrategyRulesChecker class methodsFor: 'as yet unclassified' stamp: 'mg 2/3/2013 01:55:48.729'!
isValid: aSudokuGrid
"This method checks the rules in a candidates grid."
	1 to: aSudokuGrid rowCount do:
		[:i |	| row column box | 
			row := (aSudokuGrid atRow: i).
			row detect: [:numberList | numberList isEmpty or: [numberList size = 1 and: [(row select: [:each | numberList includesAllOf: each]) size > 1]]]
				ifFound: [:temp | ^false]
				ifNone: [].
			
			column := aSudokuGrid atColumn: i.
			column detect: [:numberList | numberList isEmpty or: [numberList size = 1 and: [(column select: [:each | numberList includesAllOf: each]) size > 1]]]
				ifFound: [:temp | ^false]
				ifNone: [].
		  	
			box := aSudokuGrid atBox: i.
			box detect: [:numberList | numberList isEmpty or: [numberList size = 1 and: [(box select: [:each | numberList includesAllOf: each]) size > 1]]]
				ifFound: [:temp | ^false]
				ifNone: []].
	^ true! !

!SudokuStrategyRulesChecker methodsFor: 'solving' stamp: 'mg 12/19/2012 14:43:49.708'!
checkConstraintForBoxAt: i at: j
"This method searches all assigned numbers in the box at (i,j) and removes this numbers in all other candidate lists in the box."
	| assignedNumbers box |
	box := self getBoxCellsAt: i at: j in: inputMatrix.
	assignedNumbers := self collectAssignedNumbers: box.
	self removeAssignedNumbers: assignedNumbers from: (self getBoxCellsAt: i at: j in: candidates).! !

!SudokuStrategyRulesChecker methodsFor: 'solving' stamp: 'mg 12/19/2012 13:34:38.321'!
checkStrategyForBoxes
"(i,j) are the indices of the top left cells in each box."
	1 to: 9 by: 3 do:
		[:i | 1 to: 9 by: 3 do:
			[:j | self checkConstraintForBoxAt: i at: j]].! !

!SudokuStrategyRulesChecker methodsFor: 'solving' stamp: 'mg 12/19/2012 14:43:49.623'!
checkStrategyForColumns
"This method searches all assigned numbers per column and removes this numbers in all other candidate lists in the column."
	1 to: candidates columnCount do:
		[:column | |assignedNumbers| 
			assignedNumbers := self collectAssignedNumbers: (inputMatrix atColumn: column).
			self removeAssignedNumbers: assignedNumbers from: (candidates atColumn: column)].! !

!SudokuStrategyRulesChecker methodsFor: 'solving' stamp: 'mg 12/19/2012 14:43:49.8'!
checkStrategyForRows
"This method searches all assigned numbers per row and removes this numbers in all other candidate lists in the row."
	1 to: candidates rowCount do:
		[:row | |assignedNumbers| assignedNumbers := self collectAssignedNumbers: (inputMatrix atRow: row).
			self removeAssignedNumbers: assignedNumbers from: (candidates atRow: row)].! !

!SudokuStrategyRulesChecker methodsFor: 'solving' stamp: 'MG 11/24/2012 15:22'!
collectAssignedNumbers: aCollection
	| assignedNumbers |
	assignedNumbers := OrderedCollection new.
	aCollection collect: 
			[:candidateList | (candidateList size == 1) 
								ifTrue: [assignedNumbers add: candidateList first]].
	^ assignedNumbers ! !

!SudokuStrategyRulesChecker methodsFor: 'naming' stamp: 'mg 2/2/2013 20:26:27.583'!
description
	^ 'Check rules'! !

!SudokuStrategyRulesChecker methodsFor: 'solving' stamp: 'mg 1/10/2013 21:11:18.959'!
findHighlightCandidates
	self
		findHighlightCandidatesForRows;
		findHighlightCandidatesForColumns;
		findHighlightCandidatesForBoxes.! !

!SudokuStrategyRulesChecker methodsFor: 'solving' stamp: 'mg 1/10/2013 22:52:27.998'!
findHighlightCandidatesForBoxes
	1 to: removeCandidates boxCount do:
		[:box | | allNumbers | 	
				allNumbers := (removeCandidates atBox: box) flatten asSet. 
				(inputMatrix atBox: box) 
					select: [:cellCandidates | cellCandidates size = 1 and: [allNumbers includes: cellCandidates first]]
					thenDo: [:highlightNumber | 
							| cellIndexInBox | 
							cellIndexInBox := (inputMatrix atBox: box) indexOf: highlightNumber.  
							highlightCandidates 
								at: (SudokuGrid indexOfbox: box cell: cellIndexInBox) x   
								at: (SudokuGrid indexOfbox: box cell: cellIndexInBox) y 
								put: highlightNumber]]! !

!SudokuStrategyRulesChecker methodsFor: 'solving' stamp: 'mg 1/10/2013 17:26:45.353'!
findHighlightCandidatesForColumns
	1 to: removeCandidates columnCount do:
		[:column | | allNumbers | 	
				allNumbers := (removeCandidates atColumn: column) flatten asSet. 
				(inputMatrix atColumn: column) 
					select: [:cellCandidates | cellCandidates size = 1 and: [allNumbers includes: cellCandidates first]]
					thenDo: [:highlightNumber | 
							highlightCandidates 
								at: ((inputMatrix atColumn: column) indexOf: highlightNumber) 
								at: column
								put: highlightNumber]].! !

!SudokuStrategyRulesChecker methodsFor: 'solving' stamp: 'mg 1/10/2013 17:24:20.268'!
findHighlightCandidatesForRows	
	1 to: removeCandidates rowCount do:
		[:row | 	| allNumbers | 	
				allNumbers := (removeCandidates atRow: row) flatten asSet. 
				(inputMatrix atRow: row) 
					select: [:cellCandidates | cellCandidates size = 1 and: [allNumbers includes: cellCandidates first]]
					thenDo: [:highlightNumber | 
							highlightCandidates 
								at: row 
								at: ((inputMatrix atRow: row) indexOf: highlightNumber) 
								put: highlightNumber]].! !

!SudokuStrategyRulesChecker methodsFor: 'solving' stamp: 'mg 12/19/2012 14:43:49.54'!
removeAssignedNumbers: numbers from: aCollection
"If at least one candidate is removed, solvedCells is true."
	aCollection do: [:candidateList | (candidateList size == 1)
										ifFalse: [solvedCells := solvedCells or: [candidateList includesAnyOf: numbers]. 
											candidateList removeAllFoundIn: numbers]].! !

!SudokuStrategyRulesChecker methodsFor: 'naming' stamp: 'mg 2/2/2013 20:26:13.912'!
strategyName
	^ 'Check rules'! !

SudokuStrategy subclass: #SudokuStrategyYWing
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Sudoku-Strategies'!

!SudokuStrategyYWing methodsFor: 'solving' stamp: 'mg 2/2/2013 23:27:57.185'!
checkStrategyForBoxes
"not needed for Y-Wing"! !

!SudokuStrategyYWing methodsFor: 'solving' stamp: 'mg 2/2/2013 23:27:46.53'!
checkStrategyForColumns
"not needed for Y-Wing"! !

!SudokuStrategyYWing methodsFor: 'solving' stamp: 'mg 2/3/2013 15:30:35.165'!
checkStrategyForRows
"It is a really tough strategy!! :-)"
"This method search for thre connected field that has two possibilities with three different numbers. Each number should be placed in one of exactly two possible cells."
	inputMatrix withIndicesDo: [:candidateList :x :y | 
		candidateList size = 2 ifTrue: [ | possibleCellsRow possibleCellsBox possibleCellsColumn |
				possibleCellsRow := (inputMatrix atRow: x) 
					select: [:each | each size = 2 and: [(each difference: candidateList) size = 1]].
				possibleCellsBox := (inputMatrix atBox: (SudokuGrid boxNumberFromIndex: x@y)) 
					select: [:each | each size = 2 and: [(each difference: candidateList) size = 1]].   
				possibleCellsColumn := (inputMatrix atColumn: y) 
					select: [:each | each size = 2 and: [(each difference: candidateList) size = 1]].
				
				possibleCellsRow do: [:candidatesListRow | | sameNumber missingNumbers |
					sameNumber := (candidatesListRow intersection: candidateList) first.
					missingNumbers := (candidateList reject: [:number | number = sameNumber]) union: (candidatesListRow reject: [:number | number = sameNumber]).
					(possibleCellsBox select: [:each | each includesAllOf: missingNumbers])
						ifNotEmpty: ["All cells in the row of the box cell that lay in the box of the current cell (with candidatesListRow) and the cells in the current box that lay in the row of the row cell."
							| removeNumber currentColumn boxCell boxNumber boxIndex columns1 columns2 |
							removeNumber := (missingNumbers difference: candidateList) anyOne.
							currentColumn := (inputMatrix atRow: x) indexOf: candidatesListRow.    
							boxCell := (possibleCellsBox select: [:each | each includesAllOf: missingNumbers]) first.
							boxNumber := SudokuGrid boxNumberFromIndex: x@y.
							boxIndex := SudokuGrid indexOfbox: boxNumber cell: ((inputMatrix atBox: boxNumber) indexOf: boxCell).
							(boxIndex x = x) ifFalse: [  
								"Remove candidates in current box"
								columns1 := SudokuGrid missingLineNumbersInBoxForLine: y.
								columns1 do: [:each |
									((candidates at: x at: each) includes: removeNumber) ifTrue: [
												solvedCells := true. 
												(highlightCandidates at: x at: currentColumn) addAll: candidatesListRow.
												(highlightCandidates at: boxIndex x at: boxIndex y) addAll: missingNumbers.
												(highlightCandidates at: x at: y) addAll: candidateList.
												(candidates at: x at: each) remove: removeNumber]].
									
								"Remove candidates in the box of the row cell"
								columns2 := SudokuGrid missingLineNumbersInBoxForLine: currentColumn.
								columns2 add: currentColumn.
								columns2 do: [:each |
									((candidates at: boxIndex x at: each) includes: removeNumber) ifTrue: [
												solvedCells := true. 
												(highlightCandidates at: x at: currentColumn) addAll: candidatesListRow.
												(highlightCandidates at: boxIndex x at: boxIndex y) addAll: missingNumbers.
												(highlightCandidates at: x at: y) addAll: candidateList.
												(candidates at: boxIndex x at: each) remove: removeNumber]]]]
								
						ifEmpty: [(possibleCellsColumn select: [:each | each includesAllOf: missingNumbers])
									ifNotEmpty: ["One possible cell to remove a candidate" | removeNumber row column |
										removeNumber := (missingNumbers difference: candidateList) anyOne.
										column := (inputMatrix atRow: x) indexOf: candidatesListRow.
										row := (inputMatrix atColumn: y) indexOf: (possibleCellsColumn select: [:each | each includesAllOf: missingNumbers]) first.
										((candidates at: row at: column) includes: removeNumber) ifTrue: [
											solvedCells := true. 
											(highlightCandidates at: x at: column) addAll: candidatesListRow.
											(highlightCandidates at: row at: y) addAll: missingNumbers.
											(highlightCandidates at: x at: y) addAll: candidateList.
											(candidates at: row at: column) remove: removeNumber]]]].
							
				possibleCellsRow ifEmpty: [
					possibleCellsColumn do: [:candidateListColumn | | sameNumber missingNumbers |
						sameNumber := (candidateListColumn intersection: candidateList) first.
						missingNumbers := (candidateList reject: [:number | number = sameNumber]) union: (candidateListColumn reject: [:number | number = sameNumber]).
						(possibleCellsBox select: [:each | each includesAllOf: missingNumbers])
							ifNotEmpty: ["All cells in the column of the box cell that lay in the box of the current cell (with candidateListColumn) and the cells in the current box that lay in the column of the column cell."
								| removeNumber row boxIndex rows2 rows1 boxCell boxNumber |
								removeNumber := (missingNumbers difference: candidateList) anyOne.
								boxCell := (possibleCellsBox select: [:each | each includesAllOf: missingNumbers]) first.
								boxNumber := SudokuGrid boxNumberFromIndex: x@y.
								boxIndex := SudokuGrid indexOfbox: boxNumber cell: ((inputMatrix atBox: boxNumber) indexOf: boxCell).
								row := (inputMatrix atColumn: y) indexOf: candidateListColumn. 
								
								(boxIndex y = y) ifFalse: [   
									"Remove candidates in the box of the cell"
									rows1 := SudokuGrid missingLineNumbersInBoxForLine: row.
									rows1 add: row.
									rows1 do: [:each |								  
										((candidates at: each at: boxIndex y) includes: removeNumber) ifTrue: [
												solvedCells := true. 
												(highlightCandidates at: row at: y) addAll: candidateListColumn.
												(highlightCandidates at: boxIndex x at: boxIndex y) addAll: missingNumbers.
												(highlightCandidates at: x at: y) addAll: candidateList.
												(candidates at: each at: boxIndex y) remove: removeNumber]].
										
									"Remove candidates in current box"		
									rows2 := SudokuGrid missingLineNumbersInBoxForLine: x.
									rows2 do: [:each |
										((candidates at: each at: y) includes: removeNumber) ifTrue: [
												solvedCells := true. 
												(highlightCandidates at: row at: y) addAll: candidateListColumn.
												(highlightCandidates at: boxIndex x at: boxIndex y) addAll: missingNumbers.
												(highlightCandidates at: x at: y) addAll: candidateList.
												(candidates at: each at: y) remove: removeNumber]]]]]]
			].
		 ]  ! !

!SudokuStrategyYWing methodsFor: 'naming' stamp: 'mg 2/2/2013 21:16:21.665'!
description
	^'Y-Wing'! !

!SudokuStrategyYWing methodsFor: 'naming' stamp: 'mg 2/2/2013 21:16:26.636'!
strategyName
	^'Y-Wing'! !

Morph subclass: #SudokuGUI
	instanceVariableNames: 'grid numbers hintButton takeStepButton playButton pauseButton buttonWrapper strategyDisplay layoutWrapper slider sudokuCellDict'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Sudoku-GUI'!

!SudokuGUI methodsFor: 'initialization' stamp: 'MR 2/2/2013 17:16:13.196'!
connectSubmorphs 

	layoutWrapper addMorph: grid.
	layoutWrapper addMorph: buttonWrapper after: grid. 
	self addMorph: layoutWrapper.
	self addMorph: slider after: layoutWrapper.
	self addMorph: strategyDisplay after: slider.! !

!SudokuGUI methodsFor: 'initialization' stamp: 'MR 1/12/2013 16:53:39.154'!
createButtonWrapper

	buttonWrapper := RectangleMorph new. 
	buttonWrapper 	
		layoutPolicy: TableLayout new;
		listDirection: #leftToRight;
		hResizing: #spaceFill;
		vResizing: #spaceFill;
		borderWidth: 3;
		borderColor: Color black;
		cellInset: 3;
		color: Color black.! !

!SudokuGUI methodsFor: 'initialization' stamp: 'MR 1/12/2013 23:50:14.102'!
createButtons: sudokuSolver
	"This method creates three buttons that are used to take the next step of solving the Sudoku."
	
	self 
		createButtonWrapper;
		createTakeStepButton: sudokuSolver;
		createPlayPauseButton: sudokuSolver.
		
	buttonWrapper 
		addMorph: takeStepButton;
		addMorph: playButton after: takeStepButton.
		
	layoutWrapper addMorph: buttonWrapper after: grid. 
	! !

!SudokuGUI methodsFor: 'initialization' stamp: 'MR 1/12/2013 15:25:14.869'!
createCells
	"This method creates all sudoku cells, that contain all candidates from 1 to 9."
	| box |
	
	box := SudokuBuilder createBoxFromMorph: SudokuCandidateCell new.
	box borderWidth: 2.
	^ (SudokuBuilder createBoxFromMorph: box)! !

!SudokuGUI methodsFor: 'initialization' stamp: 'mg 6/18/2014 14:39'!
createConstraintButtons
	"This method creates a hint button."
	
	self 
		createButtonWrapper;
		createHintButton.
		
	buttonWrapper 
		addMorph: hintButton.
		
	layoutWrapper addMorph: buttonWrapper after: grid. 
	! !

!SudokuGUI methodsFor: 'initialization' stamp: 'mg 6/18/2014 14:42'!
createHintButton
	"This method creates a help button. If the use clicks on it, a random number (of a valid solution) will be given."
	hintButton := SimpleButtonMorph new.
	hintButton 
		actionSelector: #giveHint;
		target: self;
		label: 'Give a hint' font: (((TextStyle named: 'BitstreamVeraSans') 
							    				fontOfSize: 36) emphasized: 1);
		layoutPolicy: TableLayout new;
		color: Color white;
		hResizing: #spaceFill;
		vResizing: #spaceFill;
		layoutInset: 10;
		wrapCentering: #center.
	
	! !

!SudokuGUI methodsFor: 'initialization' stamp: 'MR 1/12/2013 17:41:41.884'!
createPlayPauseButton: sudokuSolver
	"This method creates a button that is used to take steps of solving the Sudoku until pause is pressed."
	
	playButton := SudokuPlayButton new.
	playButton  
		sudokuSolver: sudokuSolver;
		actionSelector: #playOrPause;
		target: playButton;
		layoutPolicy: TableLayout new;
		color: Color white;
		hResizing: #spaceFill;
		vResizing: #spaceFill;
		layoutInset: 10;
		wrapCentering: #center.
		
	playButton 
		playForm: (Form fromFileNamed: 'Play.png');
		pauseForm: (Form fromFileNamed: 'Pause.png').! !

!SudokuGUI methodsFor: 'initialization' stamp: 'MR 2/2/2013 19:15:56.552'!
createSlider: aSudokuSolver

	slider := SudokuSlider new.
	slider sudokuSolver: aSudokuSolver.
	^ slider.! !

!SudokuGUI methodsFor: 'initialization' stamp: 'MR 2/2/2013 17:15:51.111'!
createStrategyDisplay: anOrderedCollection

	strategyDisplay := SudokuStrategyDisplay new.
	strategyDisplay initializeStrategyTexts: anOrderedCollection.
	strategyDisplay highlightLine: 1.
	^strategyDisplay! !

!SudokuGUI methodsFor: 'initialization' stamp: 'MR 1/12/2013 20:10:12.713'!
createTakeStepButton: sudokuSolver
	"This method creates a button that is used to take the next step of solving the Sudoku."
	takeStepButton := SimpleButtonMorph new.
	takeStepButton 
		actionSelector: #takeStep;
		target: sudokuSolver;
		label: 'Take Step' font: (((TextStyle named: 'BitstreamVeraSans') 
							    				fontOfSize: 36) emphasized: 1);
		layoutPolicy: TableLayout new;
		color: Color white;
		vResizing: #spaceFill;
		width: 69*9;
		layoutInset: 10;
		wrapCentering: #center.! !

!SudokuGUI methodsFor: 'as yet unclassified' stamp: 'mg 6/19/2014 13:49'!
giveHint

	| randomX randomY randomGenerator number textMorph |
	(sudokuCellDict values anySatisfy: [:cell | cell submorphs first asText string = ''])
		ifFalse: [self inform: 'There are no empty cells.'. ^ self].
	randomX := 1.
	randomY := 1.
	randomGenerator := Random new.
	textMorph := (sudokuCellDict at: randomX@randomY) submorphs first.
	[textMorph asText string = ''] 
		whileFalse: [randomX := randomGenerator nextInt: 9.
					randomY := randomGenerator nextInt: 9.
					textMorph := (sudokuCellDict at: randomX@randomY) submorphs first.].
	number := self numbers at: randomX at: randomY.
	self numbers createConstraintAt: randomX at: randomY with: number. "should not fail"
	textMorph updateColor: Color red.
	textMorph updateTextWith: number.! !

!SudokuGUI methodsFor: 'initialization' stamp: 'mg 6/18/2014 14:02'!
initialize

	super initialize.
	
	self 
		initializeThisMorph;
		initializeGrid;
		initializeLayoutWrapper.
		
	layoutWrapper addMorph: grid.
	self addMorph: layoutWrapper.

	sudokuCellDict := Dictionary new.! !

!SudokuGUI methodsFor: 'initialization' stamp: 'MR 1/12/2013 16:00:16.541'!
initializeGrid 

	grid := self createCells.
	grid borderWidth: 0.! !

!SudokuGUI methodsFor: 'initialization' stamp: 'MR 1/12/2013 16:00:26.561'!
initializeLayoutWrapper

	layoutWrapper := Morph new. 
	layoutWrapper 	
		layoutPolicy: TableLayout new;
		listDirection: #topToBottom;
		hResizing: #spaceFill;
		vResizing: #spaceFill;
		layoutInset: 0;
		cellInset: 0;
		borderWidth: 0;
		color: Color orange.! !

!SudokuGUI methodsFor: 'initialization' stamp: 'MR 1/12/2013 16:28:49.124'!
initializeThisMorph

	self
		layoutPolicy: TableLayout new;
		listDirection: #leftToRight;
		hResizing: #shrinkWrap;
		vResizing: #shrinkWrap;
		borderWidth: 5;
		cellInset: 0;
		borderColor: Color black;
		color: Color black.! !

!SudokuGUI methodsFor: 'accessing' stamp: 'mg 6/18/2014 14:44'!
numbers

	^ numbers! !

!SudokuGUI methodsFor: 'accessing' stamp: 'mg 6/18/2014 14:44'!
numbers: aSudokuGrid

	numbers := aSudokuGrid.! !

!SudokuGUI methodsFor: 'accessing' stamp: 'MR 12/23/2012 18:09'!
setButtonName: aString

	takeStepButton submorphs first  contents: aString! !

!SudokuGUI methodsFor: 'updating' stamp: 'MR 2/2/2013 18:34:47.088'!
setCells: aMatrix ofType: aSymbol
	"Sets the values of the grid according to the given matrix. The values can be #number or #candidates."
	| a b c d |
	
	a := 0.
	grid submorphs do:
	[:threeBoxes | a := a + 1. b := 0. threeBoxes submorphs do:
		[:box | b := b + 1. c := 0. box submorphs do:
			[:threeCells | c := c + 1. d := 0. threeCells submorphs do:
				[:cell | d := d + 1. cell 
									updateContent: (aMatrix at: ((b-1)*3+d) at: ((a-1)*3+c)) 
									ofType: aSymbol]]]]. ! !

!SudokuGUI methodsFor: 'updating' stamp: 'mg 6/18/2014 14:02'!
setMatrixCoordinatesOfSudoku: aSudokuGrid
	"Sets the x and y coordinated of the corresponding matrix."
	| a b c d |
	
	a := 0.
	grid submorphs do:
	[:threeBoxes | a := a + 1. b := 0. threeBoxes submorphs do:
		[:box | b := b + 1. c := 0. box submorphs do:
			[:threeCells | c := c + 1. d := 0. threeCells submorphs do:
				[:cell | d := d + 1. 
						cell sudokuModel: aSudokuGrid; xCoord: ((b-1)*3+d); yCoord: ((a-1)*3+c).
						sudokuCellDict at: ((b-1)*3+d)@((a-1)*3+c) put: cell.]]]]. ! !

!SudokuGUI methodsFor: 'accessing' stamp: 'mg 6/18/2014 14:18'!
sudokuCellAtX: xCoord Y: yCoord

	^ sudokuCellDict at: xCoord@yCoord ifAbsent: [nil]! !

!SudokuGUI methodsFor: 'updating' stamp: 'MG 12/7/2012 15:18'!
updateCells: aMatrix ofType: aSymbol
	"Sets the values of the grid according to the given matrix. The values can be #number or #candidates."
	| a b c d |
	
	a := 0.
	grid submorphs do:
	[:threeBoxes | a := a + 1. b := 0. threeBoxes submorphs do:
		[:box | b := b + 1. c := 0. box submorphs do:
			[:threeCells | c := c + 1. d := 0. threeCells submorphs do:
				[:cell | d := d + 1. cell 
									updateContent: (aMatrix at: ((b-1)*3+d) at: ((a-1)*3+c)) 
									ofType: aSymbol]]]]. ! !

TextMorph subclass: #SudokuTextMorph
	instanceVariableNames: 'highlightColor removeCandidatesColor updateColor normalColor'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Sudoku-GUI'!

!SudokuTextMorph methodsFor: 'event handling' stamp: 'mg 6/18/2014 12:50'!
handlesKeyboard: evt

	^ true! !

!SudokuTextMorph methodsFor: 'event handling' stamp: 'mg 6/18/2014 12:50'!
handlesMouseOver: evt

	^ true! !

!SudokuTextMorph methodsFor: 'accessing' stamp: 'MG 12/15/2012 14:52'!
highlightColor
	"Answer the value of highlightColor"

	^ highlightColor! !

!SudokuTextMorph methodsFor: 'accessing' stamp: 'MG 12/15/2012 14:52'!
highlightColor: aColor
	"Set the value of highlightColor"

	highlightColor := aColor! !

!SudokuTextMorph methodsFor: 'initialization' stamp: 'mg 6/19/2014 11:19'!
initialize 

	super initialize.
	self 
		updateColor: Color blue;
		normalColor: Color black;
		highlightColor: Color green.! !

!SudokuTextMorph methodsFor: 'event handling' stamp: 'mg 6/19/2014 17:02'!
keyStroke: anEvent

	| numberFromAsciiInput |
	numberFromAsciiInput := anEvent keyValue - 48.
	(numberFromAsciiInput between: 1 and: 9) 
		ifTrue: ["try solving, set number if possible" 
				[self owner suggestNewValueToSudokuModel: numberFromAsciiInput] 
					ifError: [self inform: 'The input value ', numberFromAsciiInput, ' made the Sudoku unsatisfiable.'. 
							self keyboardFocusChange: false.
							^ self].
				self updateTextWith: numberFromAsciiInput.
				^ self].

	"navigate on arrow keys"
	(anEvent keyCharacter = Character arrowDown)
		ifTrue: [self owner moveFocusBy: 1@0 event: anEvent. ^ self].
	(anEvent keyCharacter = Character arrowUp)
		ifTrue: [self owner moveFocusBy: -1@0 event: anEvent. ^ self].
	(anEvent keyCharacter = Character arrowLeft)
		ifTrue: [self owner moveFocusBy: 0@(-1) event: anEvent. ^ self].
	(anEvent keyCharacter = Character arrowRight)
		ifTrue: [self owner moveFocusBy: 0@1 event: anEvent. ^ self].

	(anEvent keyCharacter = Character backspace
		or: [anEvent keyCharacter = Character delete])
			ifTrue: ["delete number in SudokuGrid"
					(self text = '') ifTrue: [^ self].
					self owner deleteNumberInSudokuModel.
					self updateColor: Color blue.
					^ self updateTextWith: ''].

	self inform: 'Invalid input for a Sudoku: ', anEvent keyString, '. Please enter a number between 1 and 9.'.
	self keyboardFocusChange: false.
	! !

!SudokuTextMorph methodsFor: 'event handling' stamp: 'mg 6/18/2014 13:38'!
mouseEnter: evt

	super mouseEnter: evt.
	evt hand newKeyboardFocus: self.
	self
		handleInteraction: [editor mouseDown: evt]
		fromEvent: evt.! !

!SudokuTextMorph methodsFor: 'accessing' stamp: 'MR 12/9/2012 23:58'!
normalColor
	"Answer the value of normalColor"

	^ normalColor! !

!SudokuTextMorph methodsFor: 'accessing' stamp: 'MR 12/9/2012 23:59'!
normalColor: aColor
	"Set the value of normalColor"

	normalColor := aColor! !

!SudokuTextMorph methodsFor: 'accessing' stamp: 'MG 12/15/2012 16:45'!
removeCandidatesColor
	"Answer the value of removeCandidatesColor"

	^ removeCandidatesColor! !

!SudokuTextMorph methodsFor: 'accessing' stamp: 'MG 12/15/2012 16:45'!
removeCandidatesColor: anObject
	"Set the value of removeCandidatesColor"

	removeCandidatesColor := anObject! !

!SudokuTextMorph methodsFor: 'visual properties' stamp: 'MR 2/1/2013 23:22:19.621'!
setToHighlightColor

	self backgroundColor: highlightColor; textColor: Color white.! !

!SudokuTextMorph methodsFor: 'visual properties' stamp: 'MR 12/9/2012 23:58'!
setToNormalColor

	self textColor: normalColor! !

!SudokuTextMorph methodsFor: 'visual properties' stamp: 'MR 2/1/2013 12:07:00.193'!
setToRemoveCandidatesColor

	self backgroundColor: removeCandidatesColor; textColor: Color white.! !

!SudokuTextMorph methodsFor: 'visual properties' stamp: 'MR 12/9/2012 23:57'!
setToUpdateColor

	self textColor: updateColor! !

!SudokuTextMorph methodsFor: 'visual properties' stamp: 'MR 2/1/2013 23:25:18.651'!
setToWhiteBackground
	self backgroundColor: Color white; textColor: normalColor.  ! !

!SudokuTextMorph methodsFor: 'accessing' stamp: 'MR 12/9/2012 23:58'!
updateColor
	"Answer the value of updateColor"

	^ updateColor! !

!SudokuTextMorph methodsFor: 'accessing' stamp: 'MR 12/9/2012 23:59'!
updateColor: aColor
	"Set the value of updateColor"

	updateColor := aColor! !

!SudokuTextMorph methodsFor: 'event handling' stamp: 'MG 12/14/2012 15:48'!
updateTextWith: aValue

	(aValue = self text asString)
		ifTrue: [self setToNormalColor]
		ifFalse: [self 
					setNumericValue: aValue;
					setToUpdateColor].! !

Slider subclass: #SudokuSlider
	instanceVariableNames: 'sudokuSolver maxRelativeValue'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Sudoku-GUI'!

!SudokuSlider methodsFor: 'initialization' stamp: 'MR 2/2/2013 19:06:22.71'!
initialize 

	super initialize.
	
	maxRelativeValue := 0.
	
	self 
		color: Color white;
		sliderColor: Color gray;
		value: 1;
		height: 758;
		width: 20.! !

!SudokuSlider methodsFor: 'accessing' stamp: 'MR 2/2/2013 19:10:08.506'!
maxRelativeValue
	"Answer the value of maxRelativeValue"

	^ maxRelativeValue! !

!SudokuSlider methodsFor: 'accessing' stamp: 'MR 2/2/2013 19:10:23.408'!
maxRelativeValue: anInteger
	"Set the value of maxRelativeValue"

	maxRelativeValue := anInteger! !

!SudokuSlider methodsFor: 'other events' stamp: 'MR 2/2/2013 19:15:03.186'!
mouseUpInSlider: anEvent

	| indexOfHistory indexAsFloatStartingWith0|
	super mouseUpInSlider: anEvent.
	
	indexAsFloatStartingWith0 := value * (maxRelativeValue - 1).
	indexOfHistory := indexAsFloatStartingWith0 // 1 + 1.
	sudokuSolver takeStepFromHistoryAt: (indexOfHistory).
	self relativeValue: indexOfHistory! !

!SudokuSlider methodsFor: 'accessing' stamp: 'MR 2/2/2013 19:26:25.241'!
relativeValue: anInteger
	| stepWidth |
	stepWidth := 1.0 / (maxRelativeValue - 1) asFloat .
	self value: anInteger - 1 * stepWidth! !

!SudokuSlider methodsFor: 'accessing' stamp: 'MR 2/2/2013 19:39:43.24'!
sliderThickness
	^100! !

!SudokuSlider methodsFor: 'accessing' stamp: 'MR 2/2/2013 19:05:18.499'!
sudokuSolver: anObject
	"Set the value of sudokuSolver"

	sudokuSolver := anObject! !

Matrix subclass: #SudokuGrid
	instanceVariableNames: 'sudokuGui solver equalityConstraints'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Sudoku-Core'!

!SudokuGrid class methodsFor: 'accessing' stamp: 'mg 5/28/2014 12:59'!
boxIndexDict
"This method returns a dictionary that associate the index of a 3x3 box with the position of the top left cell of the box."
	^ Dictionary new
		at: 1 put: 1@1;
		at: 2 put: 1@4;
		at: 3 put: 1@7;
		at: 4 put: 4@1;
		at: 5 put: 4@4;
		at: 6 put: 4@7;
		at: 7 put: 7@1;
		at: 8 put: 7@4;
		at: 9 put: 7@7;
		yourself! !

!SudokuGrid class methodsFor: 'as yet unclassified' stamp: 'mg 2/2/2013 17:05:27.422'!
boxIndexFromCellIndex: aPoint
	^ 
	(aPoint x - 1 \\ 3) 	"1,4,7 -> 0     2,5,8 -> 1    3,6,9 -> 2"
	* 3
	+ (aPoint y - 1 \\ 3)
	+ 1
	! !

!SudokuGrid class methodsFor: 'accessing' stamp: 'mg 2/2/2013 16:13:35.687'!
boxNumberFromIndex: aPoint
	| xBox yBox |
	xBox := aPoint x - 1 // 3.
	yBox := aPoint y - 1 // 3.
	^ xBox * 3 + yBox + 1! !

!SudokuGrid class methodsFor: 'instance creation' stamp: 'mg 5/28/2014 13:14'!
getCandidatesMatrixFrom: aMatrix

	| allCandidates candidates |
	allCandidates := OrderedCollection new.
	1 to: 9 do:
		[:i | allCandidates add: i].
	
	candidates := Matrix new: 9.
	1 to: 9 do:
		[:i | 1 to: 9 do:
			[:j | ((aMatrix at: i at: j) = 0)
					ifTrue: [candidates at: i at: j put: allCandidates copy]
					ifFalse: [candidates at: i at: j put: (OrderedCollection with: (aMatrix at: i at: j))]]].

	^ candidates! !

!SudokuGrid class methodsFor: 'accessing' stamp: 'mg 1/10/2013 22:47:16.215'!
indexOfbox: boxIndex cell: cellIndex
	^ ((self topLeftCellOfBox: boxIndex) x + (cellIndex-1 // 3))
		@ ((self topLeftCellOfBox: boxIndex) y + (cellIndex-1 \\ 3))! !

!SudokuGrid class methodsFor: 'as yet unclassified' stamp: 'mg 2/2/2013 23:39:40.476'!
missingLineNumbersInBoxForLine: lineNumber
	| dict |
	dict := Dictionary new.
	dict 
		add: 1 -> (OrderedCollection with: 2 with: 3);
		add: 2 -> (OrderedCollection with: 1 with: 3);
		add: 3 -> (OrderedCollection with: 1 with: 2);
		add: 4 -> (OrderedCollection with: 5 with: 6);
		add: 5 -> (OrderedCollection with: 4 with: 6);
		add: 6 -> (OrderedCollection with: 4 with: 5);
		add: 7 -> (OrderedCollection with: 8 with: 9);
		add: 8 -> (OrderedCollection with: 7 with: 9);
		add: 9 -> (OrderedCollection with: 7 with: 8).
	^ dict at: lineNumber! !

!SudokuGrid class methodsFor: 'instance creation' stamp: 'mg 6/18/2014 15:42'!
newConstrainedSudokuFromFile: aFileName

	| sudoku |
	sudoku := self newFromFile: aFileName.
	sudoku createConstraints.
	sudoku sudokuGui openInWorld.
	^ sudoku! !

!SudokuGrid class methodsFor: 'instance creation' stamp: 'mg 6/10/2014 15:46'!
newFrom: aMatrix

	^ self rows: aMatrix rowCount columns: aMatrix columnCount contents: aMatrix asArray deepCopy! !

!SudokuGrid class methodsFor: 'instance creation' stamp: 'mg 6/18/2014 14:44'!
newFromFile: aFileName
	| file numberList numbers numbersMatrix candidatesMatrix |

	numberList := OrderedCollection new.
	file := MultiByteFileStream fileNamed: aFileName.
	file upToEnd asString substrings
		collect: [:row | row do: [:digit | numberList add: digit digitValue]].

	numbers := Matrix new: 9 tabulate: [:i :j | numberList at: (j \\ 10 + (i-1*9))].
	file close.
	
	numbersMatrix := self newFrom: numbers.
	candidatesMatrix := self getCandidatesMatrixFrom: numbers.

	numbersMatrix sudokuGui: SudokuGUI new.
	"numbersMatrix sudokuGui updateCells: candidatesMatrix ofType: #candidates."
	numbersMatrix sudokuGui updateCells: numbersMatrix ofType: #number.
	numbersMatrix sudokuGui setMatrixCoordinatesOfSudoku: numbersMatrix.
	numbersMatrix sudokuGui numbers: numbersMatrix.
	numbersMatrix sudokuGui createConstraintButtons.
	"numbersMatrix sudokuGui openInWorld."

	^ numbersMatrix! !

!SudokuGrid class methodsFor: 'accessing' stamp: 'mg 5/28/2014 12:59'!
topLeftCellOfBox: index

	^ self boxIndexDict at: index! !

!SudokuGrid methodsFor: 'manipulating' stamp: 'mg 6/23/2014 13:45'!
addConstraintsForAllGivenNumbers

	self withIndicesDo: [:val :row :column | 
		val ~= 0 ifTrue: [equalityConstraints at: row at: column put: 
							([(self at: row at: column) = val] alwaysSolveWith: solver)]].! !

!SudokuGrid methodsFor: 'accessing boxes' stamp: 'mg 6/17/2014 14:45'!
atBox: index
"This method returns a new collection of all cells in the box at index."
	| box topLeftIndex |
	(index between: 1 and: nrows)
		ifFalse: [self error: 'box subscript out of range'].
	box := Array new: nrows.
	topLeftIndex := self class topLeftCellOfBox: index.
	0 to: 2 do:
		[:x | 0 to: 2 do:
			[:y | box at: (x*3 + y + 1) put: (self at: topLeftIndex x + x at: topLeftIndex y + y)]].
	^ box! !

!SudokuGrid methodsFor: 'accessing boxes' stamp: 'mg 1/10/2013 22:39:32.542'!
boxCount
	^ self rowCount! !

!SudokuGrid methodsFor: 'accessing' stamp: 'mg 6/23/2014 14:03'!
createConstraintAt: x at: y with: aValue

	| constraint |
	constraint := [(self at: x at: y) = aValue] alwaysSolveWith: solver.
	((self at: x at: y) = aValue)
		ifFalse: [self disableEqualityConstraintAt: x at: y.
				[solver solve] ifError: [constraint disable. ConstraintUnsatisfiableError new signal]].
	equalityConstraints at: x at: y put: constraint.! !

!SudokuGrid methodsFor: 'manipulating' stamp: 'mg 6/23/2014 13:43'!
createConstraints

	solver variableType: 'Int'.
	solver autoSolve: false.

	self addConstraintsForAllGivenNumbers.

	[self allSatisfy: [:val | val between: 1 and: 9]] alwaysSolveWith: solver.

	(1 to: self rowCount) do: [:index |
		[(self atRow: index) allDifferent &
		(self atColumn: index) allDifferent &
		(self atBox: index) allDifferent] alwaysSolveWith: solver].

	Transcript show: 'milliseconds to solve the sudoku constraints: ', [solver solve] timeToRun; cr.! !

!SudokuGrid methodsFor: 'manipulating' stamp: 'mg 6/23/2014 14:04'!
disableEqualityConstraintAt: x at: y

	(equalityConstraints at: x at: y) ifNil: [Transcript show: 
		'WARNING - Cannot disable the equality constraint at (', x, ',', y, '), because there is no constraint to disable.'; cr.
		^ self].
	(equalityConstraints at: x at: y) disable.
	equalityConstraints at: x at: y put: nil.! !

!SudokuGrid methodsFor: 'initialize-release' stamp: 'mg 6/23/2014 13:51'!
initialize

	super initialize.
	solver := ConstraintSolver newZ3Solver.
	equalityConstraints := Matrix new: 9.! !

!SudokuGrid methodsFor: 'accessing' stamp: 'mg 6/10/2014 13:36'!
sudokuGui

	^ sudokuGui! !

!SudokuGrid methodsFor: 'accessing' stamp: 'mg 6/10/2014 13:36'!
sudokuGui: anObject

	sudokuGui := anObject! !
